@@ -1,3 +1,8 @@
+// Copyright 2016 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
+
 #include Fetch.js
 
 var LibraryFetch = {
@@ -18,7 +23,7 @@ var LibraryFetch = {
   $__emscripten_fetch_load_cached_data: __emscripten_fetch_load_cached_data,
   $__emscripten_fetch_cache_data: __emscripten_fetch_cache_data,
   $__emscripten_fetch_xhr: __emscripten_fetch_xhr,
-  emscripten_start_fetch__deps: ['$Fetch', '$__emscripten_fetch_xhr', '$__emscripten_fetch_cache_data', '$__emscripten_fetch_load_cached_data', '$__emscripten_fetch_delete_cached_data', '_emscripten_get_fetch_work_queue', 'emscripten_is_main_runtime_thread', 'pthread_mutex_lock', 'pthread_mutex_unlock'],
+  emscripten_start_fetch__deps: ['$Fetch', '$__emscripten_fetch_xhr', '$__emscripten_fetch_cache_data', '$__emscripten_fetch_load_cached_data', '$__emscripten_fetch_delete_cached_data', '_emscripten_get_fetch_work_queue', 'emscripten_is_main_runtime_thread'],
   emscripten_start_fetch: emscripten_start_fetch
 };
 
