@@ -1,3 +1,7 @@
+// Copyright 2012 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
 
 var LibraryGLUT = {
   $GLUT__deps: ['$Browser'],
@@ -329,9 +333,13 @@ var LibraryGLUT = {
     }
   },
 
+  glutGetModifiers__proxy: 'sync',
+  glutGetModifiers__sig: 'i',
   glutGetModifiers: function() { return GLUT.modifiers; },
 
   glutInit__deps: ['$Browser'],
+  glutInit__proxy: 'sync',
+  glutInit__sig: 'vii',
   glutInit: function(argcp, argv) {
     // Ignore arguments
     GLUT.initTime = Date.now();
@@ -389,11 +397,15 @@ var LibraryGLUT = {
     });
   },
 
+  glutInitWindowSize__proxy: 'sync',
+  glutInitWindowSize__sig: 'vii',
   glutInitWindowSize: function(width, height) {
     Browser.setCanvasSize( GLUT.initWindowWidth = width,
                            GLUT.initWindowHeight = height );
   },
 
+  glutInitWindowPosition__proxy: 'sync',
+  glutInitWindowPosition__sig: 'vii',
   glutInitWindowPosition: function(x, y) {
     // Ignore for now
   },
@@ -423,12 +435,22 @@ var LibraryGLUT = {
       case 700: /* GLUT_ELAPSED_TIME */
         var now = Date.now();
         return now - GLUT.initTime;
+      case 0x0069: /* GLUT_WINDOW_STENCIL_SIZE */
+        return Module.ctx.getContextAttributes().stencil ? 8 : 0;
+      case 0x006A: /* GLUT_WINDOW_DEPTH_SIZE */
+        return Module.ctx.getContextAttributes().depth ? 8 : 0;
+      case 0x006E: /* GLUT_WINDOW_ALPHA_SIZE */
+        return Module.ctx.getContextAttributes().alpha ? 8 : 0;
+      case 0x0078: /* GLUT_WINDOW_NUM_SAMPLES */
+        return Module.ctx.getContextAttributes().antialias ? 1 : 0;
 
       default:
         throw "glutGet(" + type + ") not implemented yet";
     }
   },
 
+  glutIdleFunc__proxy: 'sync',
+  glutIdleFunc__sig: 'vi',
   glutIdleFunc: function(func) {
     function callback() {
       if (GLUT.idleFunc) {
@@ -442,46 +464,68 @@ var LibraryGLUT = {
     GLUT.idleFunc = func;
   },
 
+  glutTimerFunc__proxy: 'sync',
+  glutTimerFunc__sig: 'viii',
   glutTimerFunc: function(msec, func, value) {
     Browser.safeSetTimeout(function() { Module['dynCall_vi'](func, value); }, msec);
   },
 
+  glutDisplayFunc__proxy: 'sync',
+  glutDisplayFunc__sig: 'vi',
   glutDisplayFunc: function(func) {
     GLUT.displayFunc = func;
   },
 
+  glutKeyboardFunc__proxy: 'sync',
+  glutKeyboardFunc__sig: 'vi',
   glutKeyboardFunc: function(func) {
     GLUT.keyboardFunc = func;
   },
 
+  glutKeyboardUpFunc__proxy: 'sync',
+  glutKeyboardUpFunc__sig: 'vi',
   glutKeyboardUpFunc: function(func) {
     GLUT.keyboardUpFunc = func;
   },
 
+  glutSpecialFunc__proxy: 'sync',
+  glutSpecialFunc__sig: 'vi',
   glutSpecialFunc: function(func) {
     GLUT.specialFunc = func;
   },
 
+  glutSpecialUpFunc__proxy: 'sync',
+  glutSpecialUpFunc__sig: 'vi',
   glutSpecialUpFunc: function(func) {
     GLUT.specialUpFunc = func;
   },
 
+  glutReshapeFunc__proxy: 'sync',
+  glutReshapeFunc__sig: 'vi',
   glutReshapeFunc: function(func) {
     GLUT.reshapeFunc = func;
   },
 
+  glutMotionFunc__proxy: 'sync',
+  glutMotionFunc__sig: 'vi',
   glutMotionFunc: function(func) {
     GLUT.motionFunc = func;
   },
 
+  glutPassiveMotionFunc__proxy: 'sync',
+  glutPassiveMotionFunc__sig: 'vi',
   glutPassiveMotionFunc: function(func) {
     GLUT.passiveMotionFunc = func;
   },
 
+  glutMouseFunc__proxy: 'sync',
+  glutMouseFunc__sig: 'vi',
   glutMouseFunc: function(func) {
     GLUT.mouseFunc = func;
   },
 
+  glutSetCursor__proxy: 'sync',
+  glutSetCursor__sig: 'vi',
   glutSetCursor: function(cursor) {
     var cursorStyle = 'auto';
     switch(cursor) {
@@ -557,7 +601,9 @@ var LibraryGLUT = {
     Module['canvas'].style.cursor = cursorStyle;
   },
   
+  glutCreateWindow__proxy: 'sync',
   glutCreateWindow__deps: ['$Browser'],
+  glutCreateWindow__sig: 'ii',
   glutCreateWindow: function(name) {
     var contextAttributes = {
       antialias: ((GLUT.initDisplayMode & 0x0080 /*GLUT_MULTISAMPLE*/) != 0),
@@ -565,17 +611,25 @@ var LibraryGLUT = {
       stencil: ((GLUT.initDisplayMode & 0x0020 /*GLUT_STENCIL*/) != 0),
       alpha: ((GLUT.initDisplayMode & 0x0008 /*GLUT_ALPHA*/) != 0)
     };
+#if OFFSCREEN_FRAMEBUFFER
+    // TODO: Make glutCreateWindow explicitly aware of whether it is being proxied or not, and set these to true only when proxying is being performed.
+    GL.enableOffscreenFramebufferAttributes(contextAttributes);
+#endif
     Module.ctx = Browser.createContext(Module['canvas'], true, true, contextAttributes);
     return Module.ctx ? 1 /* a new GLUT window ID for the created context */ : 0 /* failure */;
   },
 
+  glutDestroyWindow__proxy: 'sync',
   glutDestroyWindow__deps: ['$Browser'],
+  glutDestroyWindow__sig: 'ii',
   glutDestroyWindow: function(name) {
     Module.ctx = Browser.destroyContext(Module['canvas'], true, true);
     return 1;
   },
 
+  glutReshapeWindow__proxy: 'sync',
   glutReshapeWindow__deps: ['$GLUT', 'glutPostRedisplay'],
+  glutReshapeWindow__sig: 'vi',
   glutReshapeWindow: function(width, height) {
     GLUT.exitFullscreen();
     Browser.setCanvasSize(width, height, true); // N.B. GLUT.reshapeFunc is also registered as a canvas resize callback.
@@ -586,14 +640,18 @@ var LibraryGLUT = {
     _glutPostRedisplay();
   },
 
+  glutPositionWindow__proxy: 'sync',
   glutPositionWindow__deps: ['$GLUT', 'glutPostRedisplay'],
+  glutPositionWindow__sig: 'vii',
   glutPositionWindow: function(x, y) {
     GLUT.exitFullscreen();
     /* TODO */
     _glutPostRedisplay();
   },
 
+  glutFullScreen__proxy: 'sync',
   glutFullScreen__deps: ['$GLUT', 'glutPostRedisplay'],
+  glutFullScreen__sig: 'v',
   glutFullScreen: function() {
     GLUT.windowX = 0; // TODO
     GLUT.windowY = 0; // TODO
@@ -605,12 +663,18 @@ var LibraryGLUT = {
     GLUT.requestFullscreen();
   },
 
+  glutInitDisplayMode__proxy: 'sync',
+  glutInitDisplayMode__sig: 'vi',
   glutInitDisplayMode: function(mode) {
     GLUT.initDisplayMode = mode;
   },
 
+  glutSwapBuffers__proxy: 'sync',
+  glutSwapBuffers__sig: 'v',
   glutSwapBuffers: function() {},
 
+  glutPostRedisplay__proxy: 'sync',
+  glutPostRedisplay__sig: 'v',
   glutPostRedisplay: function() {
     if (GLUT.displayFunc && !GLUT.requestedAnimationFrame) {
       GLUT.requestedAnimationFrame = true;
@@ -623,7 +687,9 @@ var LibraryGLUT = {
     }
   },
 
+  glutMainLoop__proxy: 'sync',
   glutMainLoop__deps: ['$GLUT', 'glutReshapeWindow', 'glutPostRedisplay'],
+  glutMainLoop__sig: 'v',
   glutMainLoop: function() {
     _glutReshapeWindow(Module['canvas'].width, Module['canvas'].height);
     _glutPostRedisplay();
