@@ -1,4 +1,9 @@
-# -*- encoding: utf8 -*-
+#!/usr/bin/env python2
+# coding=utf-8
+# Copyright 2013 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
 
 '''
 This tool extracts information about structs and defines from the C headers.
@@ -385,13 +390,17 @@ def inspect_code(headers, cpp_opts, structs, defines):
     cpp_opts += ['-s', 'BINARYEN=1']
 
   info = []
+  # Compile the program.
+  show('Compiling generated code...')
+  # -Oz optimizes enough to avoid warnings on code size/num locals
+  cmd = [shared.PYTHON, shared.EMCC] + cpp_opts + ['-o', js_file[1], src_file[1], '-s', 'BOOTSTRAPPING_STRUCT_INFO=1', '-s', 'WARN_ON_UNDEFINED_SYMBOLS=0', '-Oz', '--js-opts', '0', '--memory-init-file', '0', '-s', 'SINGLE_FILE=1', '-s', 'WASM=0', '-Wno-format']
+  if shared.Settings.WASM_OBJECT_FILES:
+    cmd += ['-s', 'WASM_OBJECT_FILES=1']
 
   try:
     try:
-      # Compile the program.
-      show('Compiling generated code...')
-      subprocess.check_call([shared.PYTHON, shared.EMCC] + cpp_opts + ['-o', js_file[1], src_file[1], '-s', 'BOOTSTRAPPING_STRUCT_INFO=1', '-s', 'WARN_ON_UNDEFINED_SYMBOLS=0', '-Oz', '--js-opts', '0', '--memory-init-file', '0', '-s', 'SINGLE_FILE=1', '-s', 'WASM=0'], env=safe_env) # -Oz optimizes enough to avoid warnings on code size/num locals
-    except:
+      subprocess.check_call(cmd, env=safe_env)
+    except subprocess.CalledProcessError:
       sys.stderr.write('FAIL: Compilation failed!\n')
       sys.exit(1)
 
