@@ -1,3 +1,8 @@
+# Copyright 2015 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
+
 '''
 Separates out the core asm module out of an emscripten output file.
 
@@ -25,6 +30,8 @@ else:
   # seek a pattern like (e.ENVIRONMENT), which is in the shell.js if-cascade for the ENVIRONMENT override
   import re
   m = re.search('\((\w+)\.ENVIRONMENT\)', everything)
+  if not m:
+    m = re.search('(\w+)\.arguments\s*=\s*\[\];', everything)
   assert m, 'cannot figure out the closured name of Module statically'
   closured_name = m.group(1)
   everything = everything.replace(module, closured_name + '["asm"]')
